/* HEV SAU PFM Sensing Simplified Model built by C S-Function
% Bandpass filter at 100kHz
% Author: Li Gan
% Date: 28.06.2010
% Electronic Lab, SEG, ETH, Zurich
% version: 1.0
*/

#define S_FUNCTION_NAME BP_Filter_CS
#define S_FUNCTION_LEVEL 2

/*
 * Need to include simstruc.h for the definition of the SimStruct and
 * its associated macro definitions.
 */
#include "simstruc.h"
#include "math.h"

/* First parameter: Transformer */
#define BPF_IDX 0
#define BPF_PARAM(S) ssGetSFcnParam(S,BPF_IDX)

/* Second parameter: Sampling time*/
#define SAMPLE_TIME_IDX 1
#define SAMPLE_TIME_PARAM(S) ssGetSFcnParam(S,SAMPLE_TIME_IDX)
 
#define NPARAMS 2
#define NINPUTS 2
#define NOUTPUTS 2
#define PI (3.141592653589793)

/*====================*
 * S-function methods *
 *====================*/

#define MDL_CHECK_PARAMETERS
#if defined(MDL_CHECK_PARAMETERS) && defined(MATLAB_MEX_FILE)
  /* Function: mdlCheckParameters =============================================
   * Abstract:
   *    Validate our parameters to verify they are okay.
   */
  static void mdlCheckParameters(SimStruct *S)
  {
      /* Check 1st parameter: Transformer parameter */
      {
          if (!mxIsDouble(BPF_PARAM(S))) {
              ssSetErrorStatus(S,"1st parameter to S-function must be "
                               "real \"BP Filter\"");
              return;
          }
      }
 
      /* Check 2nd parameter: Sample Time parameter */
      {
          if (!mxIsDouble(SAMPLE_TIME_PARAM(S))) {
              ssSetErrorStatus(S,"2nd parameter to S-function must be "
                               "real \"sample time\"");
              return;
          }
      }
  }
#endif /* MDL_CHECK_PARAMETERS */

/* Function: mdlInitializeSizes ===============================================
 * Abstract:
 *    The sizes information is used by Simulink to determine the S-function
 *    block's characteristics (number of inputs, outputs, states, etc.).
 */
static void mdlInitializeSizes(SimStruct *S)
{
    /* Parameters */
    ssSetNumSFcnParams(S, NPARAMS);  /* Number of expected parameters */
    #if defined(MATLAB_MEX_FILE)
    if (ssGetNumSFcnParams(S) == ssGetSFcnParamsCount(S)) {
        mdlCheckParameters(S);
        if (ssGetErrorStatus(S) != NULL) {
            return;
        }
    } else {
        return; /* Parameter mismatch will be reported by Simulink */
    }
    #endif
    
    /* Inputs */
    if (!ssSetNumInputPorts(S, NINPUTS)) return; /* Input ports:
                                                  * 0. Complex input
                                                  * 1. Noise input
                                                 */
    
    // Dynamically type 
    ssSetInputPortDataType(S, 0, DYNAMICALLY_TYPED);
    ssSetInputPortDataType(S, 1, DYNAMICALLY_TYPED);
    // Input ports width is 1
    ssSetInputPortWidth(S, 0, 1);
    ssSetInputPortWidth(S, 1, 1);
    // Accept frame or not frame data
    ssSetInputPortFrameData(S, 0, FRAME_NO);
    ssSetInputPortFrameData(S, 1, FRAME_NO);
    // Accept complex or not complex data
    ssSetInputPortComplexSignal(S, 0, COMPLEX_INHERITED);
    ssSetInputPortComplexSignal(S, 1, COMPLEX_INHERITED);
    // Direct feed through
    ssSetInputPortDirectFeedThrough(S, 0, 1);
    ssSetInputPortDirectFeedThrough(S, 1, 1);
    
    /* Outputs */
    if (!ssSetNumOutputPorts(S, NOUTPUTS)) return; // One output port

    // Dynamically type
    ssSetOutputPortDataType(S, 0, DYNAMICALLY_TYPED);
    ssSetOutputPortDataType(S, 1, DYNAMICALLY_TYPED);
    // Vector dimension 1
    if(!ssSetOutputPortVectorDimension(S, 0, 1)) return;
    if(!ssSetOutputPortVectorDimension(S, 1, 1)) return;
    // No frame output
    ssSetOutputPortFrameData(S, 0, FRAME_NO);
    ssSetOutputPortFrameData(S, 1, FRAME_NO);
    // Complex output
    ssSetOutputPortComplexSignal(S, 0, COMPLEX_YES);

    /* Number of sample time */
    ssSetNumSampleTimes(S, 1);
    
    /* specify the sim state compliance to be same as a built-in block */
    ssSetSimStateCompliance(S, USE_DEFAULT_SIM_STATE);

    ssSetOptions(S,
                 SS_OPTION_WORKS_WITH_CODE_REUSE |
                 SS_OPTION_EXCEPTION_FREE_CODE      |
                 SS_OPTION_CAN_BE_CALLED_CONDITIONALLY |
                 SS_OPTION_USE_TLC_WITH_ACCELERATOR |
                 SS_OPTION_NONVOLATILE);
}



/* Function: mdlInitializeSampleTimes =========================================
 * Abstract:
 *    This function is used to specify the sample time(s) for your
 *    S-function. You must register the same number of sample times as
 *    specified in ssSetNumSampleTimes.
 */
static void mdlInitializeSampleTimes(SimStruct *S)
{
    ssSetSampleTime(S, 0, *mxGetPr(SAMPLE_TIME_PARAM(S)));
    ssSetOffsetTime(S, 0, 0.0);
    ssSetModelReferenceSampleTimeDefaultInheritance(S);
}


/* Function: mdlOutputs =======================================================
 * Abstract:
 *    In this function, you compute the outputs of your S-function
 *    block.
 */
static void mdlOutputs(SimStruct *S, int_T tid)
{
    
    InputRealPtrsType uPtr = ssGetInputPortRealSignalPtrs(S,0);
    real_T *y = ssGetOutputPortSignal(S,0);
    real_T *yn = ssGetOutputPortSignal(S,1);
    
    const real_T *bpfpr = mxGetPr(BPF_PARAM(S));
    
    const real_T omega = 2*PI*100000; // 2*pi*100kHz
    const real_T omega2 = omega*omega; // omega^2

    /* Complex output */
    real_T  yrm = 0.0; /* real part of the HP output */
    real_T  yim = 0.0; /* imag part of the HP output */        
    real_T  yr = 0.0;  /* real part of the total output */
    real_T  yi = 0.0;  /* imag part of the total output */
    real_T  ynm = 0.0; /* noise HP filter output */
    
    /* HP parameters */
    real_T HP_R1 = bpfpr[0];
    real_T HP_R2 = bpfpr[1]; 
    real_T HP_R3 = bpfpr[2];
    real_T HP_C1 = bpfpr[3];
    real_T HP_C2 = bpfpr[4];
    real_T HP_R12 = 1/(1/HP_R1+1/HP_R2);
    
    /* LP parameters */
    real_T LP_R1 = bpfpr[5];
    real_T LP_R2 = bpfpr[6];
    real_T LP_R3 = bpfpr[7];
    real_T LP_R4 = bpfpr[8];
    real_T LP_R5 = bpfpr[9];
    real_T LP_C1 = bpfpr[10];
    real_T LP_C2 = bpfpr[11];
    real_T LP_C3 = bpfpr[12];
    real_T LP_R24 = 1/(1/LP_R2+1/LP_R4);
    real_T LP_C12 = LP_C1+LP_C2;
    real_T Gain = bpfpr[13];

    /* Transfer function parameters : HP filter */
    real_T hb2 = HP_R12*HP_R3*HP_C1*HP_C2;
    real_T ha2 = HP_R12*HP_R3*HP_C1*HP_C2;
    real_T ha1 = HP_R3*(HP_C1+HP_C2);
    real_T denh = pow((1-ha2*omega2),2)+omega2*ha1*ha1;
    
    /* Transfer function parameters : LP filter */
    real_T lb0 = -LP_R3/LP_R1;
    real_T la2 = LP_R3*LP_R24*LP_C12*LP_C3;
    real_T la1 = (LP_R24+LP_R3)*LP_C12+LP_R24*LP_R3*LP_C12/LP_R1;
    real_T denl = pow((1-la2*omega2),2)+omega2*la1*la1;
    
    /* Calculate the HP filter output */
    real_T vr = uPtr[0][0]; // Input real part
    real_T vi = uPtr[0][1]; // Input imag part

    yrm = (-vr*hb2*omega2*(1-ha2*omega2)-vi*omega2*omega*hb2*ha1)/denh;
    yim = (-vi*hb2*omega2*(1-ha2*omega2)+vr*omega2*omega*hb2*ha1)/denh;
    ynm = *uPtr[1]*omega2*hb2/(sqrt(pow((1-omega2*ha2),2)+pow(omega*ha1,2)));
    
    /* Calculate the LP filter output */
    yr = (yrm*lb0*(1-la2*omega2)+yim*omega*lb0*la1)/denl;
    yi = (yim*lb0*(1-la2*omega2)-yrm*omega*lb0*la1)/denl;
    *yn = ynm*lb0/sqrt(pow((1-omega2*la2),2)+pow(omega*la1,2));
            
    /* Update output */
    y[0] = yr*Gain;
    y[1] = yi*Gain;
}

/* Function: mdlTerminate =====================================================
 * Abstract:
 *    In this function, you should perform any actions that are necessary
 *    at the termination of a simulation.  For example, if memory was
 *    allocated in mdlStart, this is the place to free it.
 */
static void mdlTerminate(SimStruct *S)
{
}


/*======================================================*
 * See sfuntmpl_doc.c for the optional S-function methods *
 *======================================================*/

/*=============================*
 * Required S-function trailer *
 *=============================*/

#ifdef  MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-file interface mechanism */
#else
#include "cg_sfun.h"       /* Code generation registration function */
#endif
