/* HEV SAU PFM Sensing Simplified Model built by C S-Function
% Transformer, TIAs and differential amplifier at 100kHz
% Author: Li Gan
% Date: 28.06.2010
% Electronic Lab, SEG, ETH, Zurich
% version: 1.0
*/

#define S_FUNCTION_NAME Transformer_TIAs_DiffAmp_CS
#define S_FUNCTION_LEVEL 2

/*
 * Need to include simstruc.h for the definition of the SimStruct and
 * its associated macro definitions.
 */
#include "simstruc.h"
#include "math.h"

#define U(element) (*uPtrs[element])  /* Pointer to Input Port0 */

/* First parameter: Transformer */
#define TRAN_IDX 0
#define TRAN_PARAM(S) ssGetSFcnParam(S,TRAN_IDX)

/* Second parameter: TIAs */ 
#define TIAS_IDX 1
#define TIAS_PARAM(S) ssGetSFcnParam(S,TIAS_IDX)

/* Third parameter: Differential amplifier */
#define DIFFAMP_IDX 2
#define DIFFAMP_PARAM(S) ssGetSFcnParam(S,DIFFAMP_IDX)

/* Fourth parameter: Sampling time*/
#define SAMPLE_TIME_IDX 3
#define SAMPLE_TIME_PARAM(S) ssGetSFcnParam(S,SAMPLE_TIME_IDX)
 
#define NPARAMS 4
#define NINPUTS 5
#define NOUTPUTS 2
#define PI (3.141592653589793)

/*====================*
 * S-function methods *
 *====================*/

#define MDL_CHECK_PARAMETERS
#if defined(MDL_CHECK_PARAMETERS) && defined(MATLAB_MEX_FILE)
  /* Function: mdlCheckParameters =============================================
   * Abstract:
   *    Validate our parameters to verify they are okay.
   */
  static void mdlCheckParameters(SimStruct *S)
  {
      /* Check 1st parameter: Transformer parameter */
      {
          if (!mxIsDouble(TRAN_PARAM(S))) {
              ssSetErrorStatus(S,"1st parameter to S-function must be "
                               "real \"Transformer\"");
              return;
          }
      }
      
      /* Check 2nd parameter: TIAs parameter */
      {
          if (!mxIsDouble(TIAS_PARAM(S))) {
              ssSetErrorStatus(S,"2nd parameter to S-function must be "
                               "real \"TIAs\"");
              return;
          }
      }
      
      /* Check 3rd parameter: Differential amplifier parameter */
      {
          if (!mxIsDouble(DIFFAMP_PARAM(S))) {
              ssSetErrorStatus(S,"3rd parameter to S-function must be "
                               "real \"DiffAmp\"");
              return;
          }
      }
 
      /* Check 4th parameter: Sample Time parameter */
      {
          if (!mxIsDouble(SAMPLE_TIME_PARAM(S))) {
              ssSetErrorStatus(S,"4th parameter to S-function must be "
                               "real \"sample time\"");
              return;
          }
      }
  }
#endif /* MDL_CHECK_PARAMETERS */

/* Function: mdlInitializeSizes ===============================================
 * Abstract:
 *    The sizes information is used by Simulink to determine the S-function
 *    block's characteristics (number of inputs, outputs, states, etc.).
 */
static void mdlInitializeSizes(SimStruct *S)
{
    /* Parameters */
    ssSetNumSFcnParams(S, NPARAMS);  /* Number of expected parameters */
    #if defined(MATLAB_MEX_FILE)
    if (ssGetNumSFcnParams(S) == ssGetSFcnParamsCount(S)) {
        mdlCheckParameters(S);
        if (ssGetErrorStatus(S) != NULL) {
            return;
        }
    } else {
        return; /* Parameter mismatch will be reported by Simulink */
    }
    #endif
    
    /* Inputs */
    if (!ssSetNumInputPorts(S, NINPUTS)) return; /* Four input ports:
                                                  * 0. Injection voltage 
                                                  * 1. Sensing C1
                                                  * 2. Sensing C2
                                                  * 3. HR/WR mode
                                                  * 4. Noise input
                                                 */
    
    // Dynamically type 
    ssSetInputPortDataType(S, 0, DYNAMICALLY_TYPED);
    ssSetInputPortDataType(S, 1, DYNAMICALLY_TYPED);
    ssSetInputPortDataType(S, 2, DYNAMICALLY_TYPED);
    ssSetInputPortDataType(S, 3, DYNAMICALLY_TYPED);
    ssSetInputPortDataType(S, 4, DYNAMICALLY_TYPED);
    // All ports width is 1
    ssSetInputPortWidth(S, 0, 1);
    ssSetInputPortWidth(S, 1, 1);
    ssSetInputPortWidth(S, 2, 1);
    ssSetInputPortWidth(S, 3, 1);
    ssSetInputPortWidth(S, 4, 1);
    // Accept frame or not frame data
    ssSetInputPortFrameData(S, 0, FRAME_NO);
    ssSetInputPortFrameData(S, 1, FRAME_NO);
    ssSetInputPortFrameData(S, 2, FRAME_NO);
    ssSetInputPortFrameData(S, 3, FRAME_NO);
    ssSetInputPortFrameData(S, 4, FRAME_NO);
    // Accept complex or not complex data
    ssSetInputPortComplexSignal(S, 0, COMPLEX_INHERITED);
    ssSetInputPortComplexSignal(S, 1, COMPLEX_INHERITED);
    ssSetInputPortComplexSignal(S, 2, COMPLEX_INHERITED);
    ssSetInputPortComplexSignal(S, 3, COMPLEX_INHERITED);
    ssSetInputPortComplexSignal(S, 4, COMPLEX_INHERITED);
    // Direct feed through
    ssSetInputPortDirectFeedThrough(S, 0, 1);
    ssSetInputPortDirectFeedThrough(S, 1, 1);
    ssSetInputPortDirectFeedThrough(S, 2, 1);
    ssSetInputPortDirectFeedThrough(S, 3, 1);
    ssSetInputPortDirectFeedThrough(S, 4, 1);
    
    /* Outputs */
    if (!ssSetNumOutputPorts(S, NOUTPUTS)) return; // One output port

    // Dynamically type
    ssSetOutputPortDataType(S, 0, DYNAMICALLY_TYPED);
    ssSetOutputPortDataType(S, 1, DYNAMICALLY_TYPED);
    // Vector dimension 1
    if(!ssSetOutputPortVectorDimension(S, 0, 1)) return;
    if(!ssSetOutputPortVectorDimension(S, 1, 1)) return;
    // No frame output
    ssSetOutputPortFrameData(S, 0, FRAME_NO);
    ssSetOutputPortFrameData(S, 1, FRAME_NO);
    // Complex output
    ssSetOutputPortComplexSignal(S, 0, COMPLEX_YES);
    ssSetOutputPortComplexSignal(S, 1, COMPLEX_INHERITED);

    /* Number of sample time */
    ssSetNumSampleTimes(S, 1);
    
    /* specify the sim state compliance to be same as a built-in block */
    ssSetSimStateCompliance(S, USE_DEFAULT_SIM_STATE);

    ssSetOptions(S,
                 SS_OPTION_WORKS_WITH_CODE_REUSE |
                 SS_OPTION_EXCEPTION_FREE_CODE      |
                 SS_OPTION_CAN_BE_CALLED_CONDITIONALLY |
                 SS_OPTION_USE_TLC_WITH_ACCELERATOR |
                 SS_OPTION_NONVOLATILE);
}



/* Function: mdlInitializeSampleTimes =========================================
 * Abstract:
 *    This function is used to specify the sample time(s) for your
 *    S-function. You must register the same number of sample times as
 *    specified in ssSetNumSampleTimes.
 */
static void mdlInitializeSampleTimes(SimStruct *S)
{
    ssSetSampleTime(S, 0, *mxGetPr(SAMPLE_TIME_PARAM(S)));
    ssSetOffsetTime(S, 0, 0.0);
    ssSetModelReferenceSampleTimeDefaultInheritance(S);
}


/* Function: mdlOutputs =======================================================
 * Abstract:
 *    In this function, you compute the outputs of your S-function
 *    block.
 */
static void mdlOutputs(SimStruct *S, int_T tid)
{
    
    InputRealPtrsType uPtrs = ssGetInputPortRealSignalPtrs(S,0);
    real_T *y = ssGetOutputPortSignal(S,0);
    real_T *yn = ssGetOutputPortSignal(S,1);
    
    const real_T *tranpr = mxGetPr(TRAN_PARAM(S));
    const real_T *tiaspr = mxGetPr(TIAS_PARAM(S));
    const real_T *diffamppr = mxGetPr(DIFFAMP_PARAM(S));
    
    const real_T omega = 2*PI*100000; // 2*pi*100kHz
    const real_T omega2 = omega*omega; // omega^2

    /* Complex output */
    real_T  yrm = 0.0; /* real part of the TIAs output */
    real_T  yim = 0.0; /* imag part of the TIAs output */        
    real_T  yr = 0.0;  /* real part of the total output */
    real_T  yi = 0.0;  /* imag part of the total output */
    
    /* Transformer parameters */
    real_T L1 = tranpr[0];
    real_T L2 = tranpr[1];
    real_T L3 = tranpr[2];
    real_T Cp1 = tranpr[3];
    real_T Cp2 = tranpr[4];
    real_T Ca1 = tranpr[5];
    real_T Ca2 = tranpr[6];
    real_T K = tranpr[7];
    real_T M13 = K*sqrt(L1*L3);
    real_T M23 = K*sqrt(L2*L3); 
    real_T ras1 = Ca1/(Ca1+Cp1+U(1));
    real_T ras2 = Ca2/(Ca2+Cp2+U(2));
    
    /* TIA parameters */
    real_T Cd1 = tiaspr[0];
    real_T Cd2 = tiaspr[1];
    real_T Ci = tiaspr[2];
    real_T Cf = tiaspr[3];
    real_T Avo = tiaspr[4];
    real_T tau = tiaspr[5];
    
    /* Differential amplifier parameters */
    real_T R1 = diffamppr[0];
    real_T R2 = diffamppr[1];
    real_T R3 = diffamppr[2];
    real_T C3 = diffamppr[3];
    real_T C4 = diffamppr[4];
    real_T R23 = 1/(1/R2+1/R3);
    
    /* Transfer function parameters : Transformer + TIAs */
    real_T alpha = 0.5*((1+Avo)*Cf+Ci);
    real_T beta = 1+alpha*(1/Cd1+1/Cd2);
    real_T gamma = 0.5*(Cf+Ci);
    real_T zeta = 1+gamma*(1/Cd1+1/Cd2);
    real_T b2 = -Avo*(M13*ras1*U(1)-M23*ras2*U(2));
    real_T a3 = tau*(L3*gamma+M13*ras1*(U(1)+Cp1)*zeta+M23*ras2*(U(2)+Cp2)*zeta);
    real_T a2 = L3*alpha+M13*ras1*(U(1)+Cp1)*beta+M23*ras2*(U(2)+Cp2)*beta;
    real_T a1 = tau*zeta;
    real_T a0 = beta;
    
    /* Transfer function parameters : Differential amplifier */
    /* HR mode */
    real_T hb1 = R3/R1*R1*C3;
    real_T ha1 = R1*C3;
    real_T denh = 1+omega2*ha1*ha1;
    /* WR mode */
    real_T wb1 = R23*C3;
    real_T wa2 = R1*R23*C3*C4;
    real_T wa1 = R23*C3*(R1/R23+C4/C3);
    real_T denw = pow((1-omega2*wa2),2)+omega2*wa1*wa1;
    
    /* Calculate the TIAs output */
    real_T denm = pow((a0-a2*omega2),2)+omega2*pow((a1-a3*omega2),2);
    yrm = -U(0)*b2*omega2*(a0-a2*omega2)/denm;
    yim = U(0)*b2*omega2*omega*(a1-a3*omega2)/denm;
    
    /* Calculate the differential amplifier output */
    switch((int)U(3)){
        case 0 : 
        {   yr = (yrm*omega2*ha1*hb1-yim*omega*hb1)/denh;
            yi = (yim*omega2*ha1*hb1+yrm*omega*hb1)/denh;
            *yn = U(4)*omega*hb1/sqrt(omega2*ha1*ha1+1);
            break;
        }
        case 1 : 
        {   yr = (yrm*omega2*wa1*wb1-yim*omega*wb1*(1-omega2*wa2))/denw;
            yi = (yim*omega2*wa1*wb1+yrm*omega*wb1*(1-omega2*wa2))/denw;
            *yn = U(4)*omega*wb1/sqrt(pow((1-omega2*wa2),2)+pow(omega*wa1,2));
            break;
        }
        default :
        {   printf("HR/WR Mode Error!");
            break;
        }
    }
            
    /* Update output */
    y[0] = yr;
    y[1] = yi;
}

/* Function: mdlTerminate =====================================================
 * Abstract:
 *    In this function, you should perform any actions that are necessary
 *    at the termination of a simulation.  For example, if memory was
 *    allocated in mdlStart, this is the place to free it.
 */
static void mdlTerminate(SimStruct *S)
{
}


/*======================================================*
 * See sfuntmpl_doc.c for the optional S-function methods *
 *======================================================*/

/*=============================*
 * Required S-function trailer *
 *=============================*/

#ifdef  MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-file interface mechanism */
#else
#include "cg_sfun.h"       /* Code generation registration function */
#endif
