%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% HEV SAU PFM Sensing test using simulink
% ADC with Noises
% Author: Li Gan
% Date: 23.04.2010
% Electronic Lab, SEG, ETH, Zurich
% version: 1.1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [sys,x0,str,ts] = ADC_Noise(t,x,u,flag,NB,V_Ref,Neg_Out,TS)
% ADC model with reference voltage noise
% The parameters include: 
% 1.NB     : Number of converting bits
% 2.V_Ref  : Reference voltage
% 3.Var_N  : Variance of the noise at the reference voltage
% 4.TS     : Sampling time

switch flag,

  % Initialization %
  case 0,
    [sys,x0,str,ts]=mdlInitializeSizes(TS);

  % Update %
  case 2,
    sys=mdlUpdate(t,x,u,NB,V_Ref,Neg_Out);

  % Outputs %
  case 3,
    sys=mdlOutputs(t,x,u);
    
  % Unused flags %
  case {1, 4, 9},
    sys=[]; 

  %%%%%%%%%%%%%%%%%%%%
  % Unexpected flags %
  %%%%%%%%%%%%%%%%%%%%
  otherwise
    error(['Unhandled flag = ',num2str(flag)]);

end

% end LineDriver

%
%=============================================================================
% mdlInitializeSizes
% Return the sizes, initial conditions, and sample times for the S-function.
%=============================================================================
%
function [sys,x0,str,ts]=mdlInitializeSizes(TS)

sizes = simsizes;
sizes.NumContStates  = 0;
sizes.NumDiscStates  = 1;
sizes.NumOutputs     = 1;
sizes.NumInputs      = 3;
sizes.DirFeedthrough = 0;
sizes.NumSampleTimes = 1;   % At least one sample time is needed
sys = simsizes(sizes);

%
% initialize the initial conditions
%
x0  = 0;

%
% str is always an empty matrix
%
str = [];

%
% initialize the array of sample times
%
ts  = [TS 0];

% end mdlInitializeSizes


%=============================================================================
% mdlUpdate
% Handle discrete state updates, sample time hits, and major time step
% requirements.
%=============================================================================
%
function sys=mdlUpdate(t,x,u,NB,V_Ref,Neg_Out)

Input = u(1,1); % Input voltage
Ref_Noise = u(2,1); % Reference noise voltage
Res_Noise = u(3,1); % Resolution noise
V_Max = V_Ref+Ref_Noise; % Max input voltage at low output (unreachable)
V_Min = -(V_Ref+Ref_Noise); % Min input voltage at low output
LSB = (V_Max-V_Min)/2^NB; % Least significant bit
Input_Noise = Input+Res_Noise;

% Saturation
if Input_Noise > V_Max-1.5*LSB
   Input_Noise = V_Max-1.5*LSB;
elseif Input_Noise < V_Min
   Input_Noise = V_Min;
end

% Output
if Neg_Out == 0
   Output = floor((Input_Noise-V_Min)/LSB+0.5);
else
   Output = floor((Input_Noise-V_Min)/LSB+0.5)-2^(NB-1);
end

x = Output;
sys=x;

% end mdlUpdate

%
%=============================================================================
% mdlOutputs
% Return the block outputs.
%=============================================================================
%
function sys=mdlOutputs(t,x,u)

sys = x;

% end mdlOutputs
