%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% HEV SAU PFM Sensing Simplified Model
% Band pass filter at 100kHz
% Author: Li Gan
% Date: 22.06.2010
% Electronic Lab, SEG, ETH, Zurich
% version: 1.1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [sys,x0,str,ts] = BP_Filter_MS(t,x,u,flag,BPF,TS)
% Band pass filter at 100kHz
% The parameters include: 
% 1.BPF : BP filter circuit parameters
% 2.TS  : Sampling time

switch flag,

  % Initialization %
  case 0,
    [sys,x0,str,ts]=mdlInitializeSizes(TS);

  % Update %
  case 2,
    sys=mdlUpdate(t,x,u,BPF);

  % Outputs %
  case 3,
    sys=mdlOutputs(t,x,u);
    
  % Unused flags %
  case {1, 4, 9},
    sys=[]; 

  %%%%%%%%%%%%%%%%%%%%
  % Unexpected flags %
  %%%%%%%%%%%%%%%%%%%%
  otherwise
    error(['Unhandled flag = ',num2str(flag)]);

end

% end LineDriver

%
%=============================================================================
% mdlInitializeSizes
% Return the sizes, initial conditions, and sample times for the S-function.
%=============================================================================
%
function [sys,x0,str,ts]=mdlInitializeSizes(TS)

sizes = simsizes;
sizes.NumContStates  = 0;
sizes.NumDiscStates  = 5;
sizes.NumOutputs     = 5;
sizes.NumInputs      = 3;
sizes.DirFeedthrough = 0;
sizes.NumSampleTimes = 1;   % At least one sample time is needed
sys = simsizes(sizes);

%
% initialize the initial conditions
%
Amp = 0;
Phase = 0;
Real = 0;
Imag = 0;
Noise = 0;

% initialize the discrete states
x0  = [Amp Phase Real Imag Noise];

%
% str is always an empty matrix
%
str = [];

%
% initialize the array of sample times
%
ts  = [TS 0];

% end mdlInitializeSizes


%=============================================================================
% mdlUpdate
% Handle discrete state updates, sample time hits, and major time step
% requirements.
%=============================================================================
%
function sys=mdlUpdate(t,x,u,BPF)

Vr = u(1,1); % Input real part
Vi = u(2,1); % Input imag part
Vin = Vr+i*Vi;
Noise_in = u(3,1); % Noise input

% Active HP
HP_R1 = BPF.HP_R1; % 22.6e03
HP_R2 = BPF.HP_R2; % 71.5e03
HP_R3 = BPF.HP_R3; % 178
HP_C1 = BPF.HP_C1; % 1e-9
HP_C2 = BPF.HP_C2; % 1e-9
HP_R12 = 1/(1/HP_R1+1/HP_R2);

% transfer function of HP filter
b1 = HP_R12*HP_R3*HP_C1*HP_C2;
a1 = [HP_R12*HP_R3*HP_C1*HP_C2 HP_R3*(HP_C1+HP_C2) 1];

% Active LP
LP_R1 = BPF.LP_R1; % 3.16e03
LP_R2 = BPF.LP_R2; % 11.5e03
LP_R3 = BPF.LP_R3; % 976
LP_R4 = BPF.LP_R4; % 93.1e03
LP_C1 = BPF.LP_C1; % 10e-12
LP_C2 = BPF.LP_C2; % 10e-12
LP_C3 = BPF.LP_C3; % 10e-9
LP_R24 = 1/(1/LP_R2+1/LP_R4);
LP_C12 = LP_C1+LP_C2;

b2 = -LP_R3/LP_R1;
a2 = [LP_R3*LP_R24*LP_C12*LP_C3 (LP_R24+LP_R3)*LP_C12+LP_R24*LP_R3*LP_C12/LP_R1 1];

% Extra gain
Gain = BPF.Gain; % 1.1783

omega = 2*pi*100e03; % Injection signal frequency 100kHz
s = 1i*omega;

V_HP_out = Vin*b1*s^2/(a1(1)*s^2+a1(2)*s+a1(3));
V_LP_out = V_HP_out*b2/(a2(1)*s^2+a2(2)*s+a2(3));
V_out = V_LP_out*Gain;

Amp = abs(V_out);
Phase = angle(V_out);
Real = real(V_out);
Imag = imag(V_out);

Noise = Noise_in*abs(b1*s^2/(a1(1)*s^2+a1(2)*s+a1(3)))...
      *abs(b2/(a2(1)*s^2+a2(2)*s+a2(3)))*Gain;

x = [Amp Phase Real Imag Noise];

sys=x;

% end mdlUpdate

%
%=============================================================================
% mdlOutputs
% Return the block outputs.
%=============================================================================
%
function sys=mdlOutputs(t,x,u)

sys = x;

% end mdlOutputs
