%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% HEV SAU PFM Sensing Simplified Model built by M-File S-Function
% Transformer, TIAs and differential amplifier at 100kHz
% Author: Li Gan
% Date: 22.06.2010
% Electronic Lab, SEG, ETH, Zurich
% version: 1.1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [sys,x0,str,ts] = Transformer_TIAs_DiffAmp_MS(t,x,u,flag,...
                                               Transformer,TIAs,DiffAmp,TS)
% Transformer, TIAs and differential amplifier at 100kHz
% The parameters include: 
% 1.Transformer : Transformer circuit parameters
% 2.TIAs        : TIAs circuit parameters
% 3.DiffAmp     : Differential amplifier circuit parameters
% 4.TS          : Sampling time

switch flag,

  % Initialization %
  case 0,
    [sys,x0,str,ts]=mdlInitializeSizes(TS);

  % Update %
  case 2,
    sys=mdlUpdate(t,x,u,Transformer,TIAs,DiffAmp);

  % Outputs %
  case 3,
    sys=mdlOutputs(t,x,u);
    
  % Unused flags %
  case {1, 4, 9},
    sys=[]; 

  %%%%%%%%%%%%%%%%%%%%
  % Unexpected flags %
  %%%%%%%%%%%%%%%%%%%%
  otherwise
    error(['Unhandled flag = ',num2str(flag)]);

end

% end LineDriver

%
%=============================================================================
% mdlInitializeSizes
% Return the sizes, initial conditions, and sample times for the S-function.
%=============================================================================
%
function [sys,x0,str,ts]=mdlInitializeSizes(TS)

sizes = simsizes;
sizes.NumContStates  = 0;
sizes.NumDiscStates  = 5;
sizes.NumOutputs     = 5;
sizes.NumInputs      = 5;
sizes.DirFeedthrough = 0;
sizes.NumSampleTimes = 1;   % At least one sample time is needed
sys = simsizes(sizes);

%
% initialize the initial conditions
%
Amp = 0;
Phase = 0;
Real = 0;
Imag = 0;
Noise = 0;

% initialize the discrete states
x0  = [Amp Phase Real Imag Noise];

%
% str is always an empty matrix
%
str = [];

%
% initialize the array of sample times
%
ts  = [TS 0];

% end mdlInitializeSizes


%=============================================================================
% mdlUpdate
% Handle discrete state updates, sample time hits, and major time step
% requirements.
%=============================================================================
%
function sys=mdlUpdate(t,x,u,Transformer,TIAs,DiffAmp)

V_inj = u(1,1); % Injection voltage level
C1 = u(2,1); % Sensing capacitance 1
C2 = u(3,1); % Sensing capacitance 2
Mode = u(4,1); % HR/WR mode
Noise_in = u(5,1); % Noise signal

% transformer+TIAs parameters
L1 = Transformer.L1; % Transformer coil inductance (4.2e-3)
L2 = Transformer.L2; % Transformer coil inductance (4.2e-3)
L3 = Transformer.L3; % Transformer coil inductance (4.2e-3)
K = Transformer.K; % Coil coupling coefficient (1)
Cp1 = Transformer.Cp1; % Resonance tuning capacitance (300.3e-12)
Cp2 = Transformer.Cp2; % Resonance tuning capacitance (300.3e-12)
Ca1 = Transformer.Ca1; % Actuation capacitance (10e-9)
Ca2 = Transformer.Ca2; % Actuation capacitance (10e-9)

Cd1 = TIAs.Cd1; % Decoupling capacitance (10e-9)
Cd2 = TIAs.Cd2; % Decoupling capacitance (10e-9)
Ci = TIAs.Ci; % TIA input capacitance (14e-12) --- U440 3e-12 U421 1.3e-12
Rf = TIAs.Rf; % Feedback resistance (5.6e06)
Cf = TIAs.Cf; % Feedback capacitance (3.3e-12)
A = TIAs.A; % OpAmp large signal voltage gain (10^(80/20))
T = TIAs.Tao; % Time constant for A corner frequency (1/(2*pi*4e03))

M13 = K*sqrt(L1*L3);
M23 = K*sqrt(L2*L3);
ras1 = Ca1/(Ca1+C1+Cp1);
ras2 = Ca2/(Ca2+C2+Cp2);

% transfer function of transformer+TIAs
b1 = [-A*M13; -A*M23]; % Numerator
a1 = [(0.5*L3*(Cf+Ci)+M13*ras1*(C1+Cp1)*(1+0.5*(Cf+Ci)*(1/Cd1+1/Cd2))+M23*ras2*(C2+Cp2)*(1+0.5*(Cf+Ci)*(1/Cd1+1/Cd2)))*T; ...
     0.5*L3*Cf*(1+A)+M13*ras1*(C1+Cp1)*(1+0.5*(Cf*(1+A)+Ci)*(1/Cd1+1/Cd2))+M23*ras2*(C2+Cp2)*(1+0.5*(Cf*(1+A)+Ci)*(1/Cd1+1/Cd2)); ...
     (0.5*(Cf+Ci)*(1/Cd1+1/Cd2)+1)*T; ...
     0.5*(Cf*(1+A)+Ci)*(1/Cd1+1/Cd2)+1]; % Denominator

% differential amplifier parameters
R1 = DiffAmp.R1; % 976
R2 = DiffAmp.R2; % 944
R3 = DiffAmp.R3; % 18.7e03
R23 = 1/(1/R2+1/R3);
C3 = DiffAmp.C3; % 10e-9
C4 = DiffAmp.C4; % 100e-12

% transfer function of differential amplifier
if Mode == 0 % HR mode
   b2 = R3*C3;
   a2 = [R1*C3; 1];
else % WR mode
   b2 = R23*C3;
   a2 = [R1*R23*C3*C4; R23*C3*(R1/R23+C4/C3); 1];   
end

omega = 2*pi*100e03; % Injection signal frequency 100kHz
s = 1i*omega;

% 1. claculate the output
V_TIA_out =  V_inj*(b1(1)*C1-b1(2)*C2)*s^2/(a1(1)*s^3+a1(2)*s^2+a1(3)*s+a1(4));
if Mode == 0 % HR mode
   V_out = V_TIA_out*b2*s/(a2(1)*s+a2(2));
else % WR mode
   V_out = V_TIA_out*b2*s/(a2(1)*s^2+a2(2)*s+a2(3));
end

Amp = abs(V_out);
Phase = angle(V_out);
Real = real(V_out);
Imag = imag(V_out);

% 2. calculate the noise output
if Mode == 0 % HR mode
   Noise = Noise_in*abs(b2*s/(a2(1)*s+a2(2)));
else % WR mode
   Noise = Noise_in*abs(b2*s/(a2(1)*s^2+a2(2)*s+a2(3)));
end
x = [Amp Phase Real Imag Noise];

sys=x;

% end mdlUpdate

%
%=============================================================================
% mdlOutputs
% Return the block outputs.
%=============================================================================
%
function sys=mdlOutputs(t,x,u)

sys = x;

% end mdlOutputs
